<#  
.SYNOPSIS  
		Returns current connection count for all frontend servers in a given pool
		including a breakdown of connection by client, frontend server and users.
 
		It can also be used to return connection information on an individual user.

.DESCRIPTION  
		This program will return a connection count for a given pool.  The program
		can be edited to set a default pool. You will also be able to get
		information on an individual user by providing the users SamAccountName.
		As well as listing all the connected users to the default pool.
		
		NOTE: In order to gain remote access to each frontend server's 
		RTCLOCAL database where connection information is found, 
		you need to open the local firewall for port 1434.
		Also, need to go into the SQL Server Configuration Manager 
		and for RTCLOCAL, enable named pipes and restart the SQL 
		service for the named pipes to take effect.

		Port 1434 is required in order to make the connection to 
		the named instance RTCLOCAL on the remote machines.

.NOTES  
    Version      	   	: 1.0 - 07/01/2011 
    									: 1.1 - 08/09/2011 - added comment based help; added option to display user list; updated formatting   
		Wish list					: Could add # users who are enterprise voice enabled and other individual stats to this page as well
    Rights Required		: TBD
    Sched Task Req'd	: No
    Lync Version			: 2010 through CU2
    Author(s)    			: Tracy A. Cerise (tracy@uky.edu)
    									: Mahmoud Badran (v-mabadr@microsoft.com)
    									: Pat Richard (pat@innervation.com)
    Dedicated Post		: 1.0 - http://blogs.technet.com/b/csps/archive/2011/03/22/scriptconnections.aspx
    Disclaimer   			: You running this script means you won't blame me if this breaks your stuff.
    Info Stolen from 	: http://blogs.technet.com/b/meacoex/archive/2011/07/19/list-connections-and-users-connected-to-lync-registrar-pool.aspx
    Acknowledgement		: This program's database connection information was originally taken from the "List Connections to Registrar Pools" submitted by Scott Stubberfield and Nick Smith from Microsoft to the Lync 2010 PowerShell blog  (http://blogs.technet.com/b/csps/) on June 10, 2010.

.LINK  
    http://blogs.technet.com/b/csps/archive/2011/03/22/scriptconnections.aspx

.EXAMPLE
		.\connections.ps1 
 
		Description
		-----------
		Returns information on all connections on all frontend servers in the 
		default pool that has been hard-coded into the program.

.EXAMPLE
		.\connections.ps1 -Pool alternate.pool.fqdn
 
		Description
		-----------
		Returns information on all connections on all frontend servers in the 
		pool FQDN given with the pool parameter.

.EXAMPLE
		.\connections.ps1 -Pool poolname
 
		Description
		-----------
		Returns information on all connections on all frontend servers in the 
		pool NetBIOS name given with the pool parameter. The script will append
		the %userdnsdomain% to build the FQDN.
		
.EXAMPLE
		.\connections.ps1 -SIPAddr userid@sip.domain
 
		Description
		-----------
		Returns all connection information for the given user including which 
		frontend server connected to, how many connections and which clients
		connected with.
 
.EXAMPLE
		.\connections.ps1 -FilePath c:\path\to\file\filename.csv
 
		Description
		-----------
		Returns information on all connections on all frontend servers in the 
		default pool that has been hard-coded into the program and in addition
		writes out all the raw connection information into the filename specified.

.EXAMPLE
		.\connections.ps1 -IncludeUsers
 
		Description
		-----------
		Returns information on all connections on all frontend servers in the 
		default pool that has been hard-coded into the program and in addition
		writes out all the users connection information.

#Requires -Version 2.0
#>
#################################################################################################
#################### Commandline Parameters to use for running this program #####################
#################################################################################################
param(
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No pool FQDN specified")] 
	[string]$Pool, 
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false, HelpMessage="No SIP address specified")] 
	[string]$SIPAddr, 
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false, HelpMessage="No file path specified")] 
	[string]$FilePath, 
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false)] 
	[switch]$IncludeUsers
)
 
#################################################################################################
##########################################  Functions  ##########################################
#################################################################################################
function Get-Data { 
	param ($sipAddr = $null, $server)
 
	##############################################################################################
	# Went to using a named parameter for this function due to the 
	# way Powershell does its thing with parameter passing, which 
	# is NOT GOOD!  At any rate, need to call this function
	# as you would from a command line: Get-Data -sipAddr "value" 
	# -server "value"
	#
	# Also, assuming a value of NULL for the SIP address of an 
	# individual user, mostly to use this for finding overall 
	# values, only occasionally to seek specific users.
	##############################################################################################
 
	if ($sipAddr) {
		$whereClause = "where R.UserAtHost = '$sipAddr' "
	} else {
		$whereClause = $null
	}
 
	#Define SQL Connection String
	$connstring = "server=$server\rtclocal;database=rtcdyn;trusted_connection=true;"
 
	#Define SQL Command
	$command = New-Object System.Data.SqlClient.SqlCommand
 
	$command.CommandText = "Select (cast (RE.ClientApp as varchar (100))) as ClientVersion, R.UserAtHost as UserName, Reg.Fqdn `
									From rtcdyn.dbo.RegistrarEndpoint RE `
									Inner Join `
									rtc.dbo.Resource R on R.ResourceId = RE.OwnerId `
									Inner Join `
									rtcdyn.dbo.Registrar Reg on Reg.RegistrarId = RE.PrimaryRegistrarClusterId `
									$whereClause `
									Order By ClientVersion, UserName "
 
	$connection = New-Object System.Data.SqlClient.SqlConnection
	$connection.ConnectionString = $connstring
	$connection.Open()
 
	$command.Connection = $connection
 
	$sqladapter = New-Object System.Data.SqlClient.SqlDataAdapter
	$sqladapter.SelectCommand = $command
 
	$results = New-Object System.Data.Dataset
 
	$recordcount=$sqladapter.Fill($results)
 
	$connection.Close()
 
	return $Results.Tables[0]
} # end function Get-Data

# make sure the Lync module is loaded (in case the script is running
# from an ordinary PowerShell console)
function Get-ModuleStatus { 
	param	(
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No module name specified!")] 
		[string]$name
	)
	if(!(Get-Module -name "$name")) { 
		if(Get-Module -ListAvailable | ? {$_.name -eq "$name"}) { 
			Import-Module -Name "$name" 
			# module was imported
			return $true | Out-Null
		} else {
			# module was not available
			return $false | Out-Null
		}
	}else {
		# module was already imported
		# Write-Host "$name module already imported"
		return $true | Out-Null
	}
} # end function Get-ModuleStatus

# let's clean up some variables at the end so we free up resources
function Remove-ScriptVariables($path) {  
	$result = Get-Content $path |  
	ForEach { if ( $_ -match '(\$.*?)\s*=') {      
			$matches[1]  | ? { $_ -notlike '*.*' -and $_ -notmatch 'result' -and $_ -notmatch 'env:'}  
		}  
	}  
	ForEach ($v in ($result | Sort-Object | Get-Unique)){		
		# Write-Host "Removing" $v.replace("$","")
		Remove-Variable ($v.replace("$","")) -ErrorAction SilentlyContinue
	}
} # end function Get-ScriptVariables
#################################################################################################
########################################  Main Program  #########################################
#################################################################################################
# Here is where we pull all the frontend server(s) from our topology for the designated
# pool and iterate through them to get current connections from all the servers.
#
# There are three possibilities here:
#	 1. Have collection of frontend servers
#	 2. Have a single frontend server or
#	 3. Have no servers
#
# Only need to check for the first two cases, the third is implied otherwise...
Get-ModuleStatus -name Lync

# set FQDN if $pool was set to just the netbiod name
if (!($pool -match $env:userdnsdomain)){$pool=$pool+"."+$env:userdnsdomain}
$feServers = Get-CsComputer -Pool $Pool
 
if ($feServers.count) {
	# Frontend pool collection, iterate through them
	for ($i=0; $i -lt $feServers.count; $i++) {

		if ($SIPAddr) {
			$data = Get-Data  -sipAddr $SIPAddr -server $feServers[$i].identity
		} else {
			$data = Get-Data -server $feServers[$i].identity
		}
 
		# Since an individual user's connections are all homed on one server, 
		# we won't have data coming back from all frontend servers in the 
		# case of searching for a single user
		if ($data) {
			$overallrecords = $overallrecords + $data
		}
	}
} elseif ($feServers) {
	# Have a standalone server or a FE pool of only one server
	if ($SIPAddr) {
		$data = Get-Data -sipAddr $SIPAddr -server $feServers.identity
	} else {
		$data = Get-Data -server $feServers.identity
	}
 
	# Make sure we have data to work with...
	if ($data) {
		$overallrecords = $data
	}
}
 
# Check to see if we have any data to act on
if (! $overallrecords) {
	Write-Host -ForegroundColor Yellow "`r`nNothing returned from query!`r`n"
 
	# Nothing else to do
	exit
} else {
	$count = 0 
	$userHash = @{}
	$clientHash = @{}
	$serverHash = @{}
	$userlist = @{}
 
	$overallrecords | ForEach-object {
		# Each record has three components: Connected Client Version, User's SIP
		# address and the frontend server's FQDN.  Here, we'll build a hash
		# for each of these components for each record.
 
		# Build hash of users

 		$userlist = ($_.UserName)

		if (! $userHash.ContainsKey($_.UserName)) {
			$userHash.add($_.UserName, 1)
		} else {
			$userHash.set_item($_.UserName, ($userHash.get_item($_.UserName) + 1))
		}
 
		# Build hash of servers
		if (! $serverHash.ContainsKey($_.fqdn)) {
			$serverHash.add($_.fqdn, 1)
		} else {
			$serverHash.set_item($_.fqdn, ($serverHash.get_item($_.fqdn) + 1))
		}
 
		# Build hash of clients
		# Lets get rid of the extraneous verbage from the client version names, if applicable
		if ($_.ClientVersion.contains('(')) {
			# Get rid of extraneous verbage
			$clientName = $_.ClientVersion.substring(0, $_.ClientVersion.IndexOf('('))
		} else {
			# Have a client name with no extraneous verbage
			$clientName = $_.ClientVersion
		}
 
		if (! $clientHash.ContainsKey($clientName)) {
			$clientHash.add($clientName, 1)
		} else {
			$clientHash.set_item($ClientName, ($clientHash.get_item($ClientName) + 1))
		}
		$count++
	}
}
 
#################################################################################################
####################################  Output Query Results  #####################################
#################################################################################################
# If output to file is chosen, then write out the results and a note to that effect
# then exit
 
if ($FilePath) {
	$overallrecords | Export-Csv $FilePath 
	Write-Host -ForegroundColor green "`r`nQuery Results written to $FilePath`r`n" 
	exit
}

# Client Version, Agent, Connections
Write-Host "`r`n`r`n"
Write-Host ("{0, -22}{1, -20}{2, 14}" -f "Client Version", "Agent", "Connections") -ForegroundColor cyan 
Write-Host -ForegroundColor cyan "--------------------------------------------------------"
ForEach ($key in $clientHash.keys) {
	# Break down client version into its two component parts and print
	# them out along with their respective counts in a nice format
	$index = $key.indexof(" ")
 
	if ($index -eq "-1") {
		# No second part
		$first = $key
		$second = " "
	} else {
		# Client version/agent has to main parts
		$first = $key.substring(0, $index)
		$second = $key.substring($index + 1)
	}
 
	$value = $clientHash.$key
	"{0,-22}{1,-21}{2,9}" -f $first, $second, $value
}
Write-Host -ForegroundColor cyan "--------------------------------------------------------"

# Frontend Server, Connections
Write-Host "`r`n`r`n"
Write-Host ("{0,-41}{1,15}" -f "Frontend Server", "Connections") -foregroundcolor cyan 
Write-Host -ForegroundColor cyan "--------------------------------------------------------"
ForEach ($key in $serverHash.keys) {
	$value = $serverHash.$key
	# [string]$Percent = "("+([Math]::Round($value/$count*100,2))+"%)"
	[string]$Percent = "("+"{0:P2}" -f ($value/$count)+")"
	"{0,-40}{1,6} {2,9}" -f $key, $value, $Percent
}
Write-Host -ForegroundColor cyan "--------------------------------------------------------"
"{0,-40}{1,6}" -f "Total connections", $count 

# Unique Users, Unique Clients
# Write-Host -ForegroundColor cyan "`r`n`r`nTotal Unique Users/Clients"
Write-Host "`r`n`r`n"
Write-Host ("{0,-41}{1,15}" -f "Total Unique Users/Clients", "Total") -foregroundcolor cyan 
Write-Host -ForegroundColor cyan "--------------------------------------------------------"
"{0,-41}{1,15}" -f "Unique Users", $userHash.count
"{0,-41}{1,15}" -f "Client Versions", $clientHash.count
Write-Host -ForegroundColor cyan "--------------------------------------------------------"

# Users, Connections
if ($IncludeUsers){
	Write-Host "`r`n`r`n"
	Write-Host ("{0,-32}{1,-5}" -f "Connected User", "Connections") -ForegroundColor cyan
	Write-Host -ForegroundColor cyan "--------------------------------------------------------"
	ForEach ($key in $userHash.keys) {
		$value = $userHash.$key
if ($value -ge 3){
 		Write-Host ("{0,-37}{1,-5}" -f $key, $value) -ForegroundColor red
}else{
	"{0,-37}{1,-5}" -f $key, $value
}
	}
	Write-Host -ForegroundColor cyan "--------------------------------------------------------`r`n"
}

Write-Host -ForegroundColor Green "Query complete`r`n"

Remove-ScriptVariables($MyInvocation.MyCommand.Name)